#!/usr/bin/env python

"""
Error classes.

Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

class ProcessingError(Exception):

    "A processing error."

    pass

class TableError(ProcessingError):

    "An error occurring during access to a lookup table."

    pass

class TableGenerationError(ProcessingError):

    "An error occurring when generating a lookup table."

    pass

class NodeProcessingError(ProcessingError):

    "A processing error associated with a particular program node."

    def __init__(self, message):
        self.message = message
        self.unit_name = None
        self.astnode = None

    def get_lineno(self, node):
        if node is None:
            return None

        lineno = node.lineno
        if lineno is not None:
            return lineno
        else:
            for child in node.getChildNodes():
                lineno = self.get_lineno(child)
                if lineno is not None:
                    return lineno
        return None

    def __repr__(self):
        return "Error in %r at line %r: %s" % (self.unit_name, self.get_lineno(self.astnode), self.message)

    def __str__(self):
        return repr(self)

class InspectError(NodeProcessingError):

    "An error during the module inspection process."

    pass

class TranslateError(NodeProcessingError):

    "An error during the module translation process."

    pass

class TranslationNotImplementedError(TranslateError):

    "An error caused by a node not being supported in translation."

    pass

# vim: tabstop=4 expandtab shiftwidth=4
